package rfd.model.routefinder.test;

import static org.junit.jupiter.api.Assertions.*;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import org.junit.jupiter.api.Test;

import rfd.model.RailwayLine;
import rfd.model.Route;
import rfd.model.Segment;
import rfd.model.Station;
import rfd.model.routefinder.ClassicRouteFinder;
import rfd.model.routefinder.RouteFinder;

public class ClassicRouteFinderTest {

	@Test
	void testOKRouteOnTheSameLine() {
		RailwayLine lineaBoLe = new RailwayLine(Map.of(
				"Santarcangelo di Romagna", new Station("Santarcangelo di Romagna", 101.27, 120),
				"RiminiFiera", new Station("RiminiFiera", 106.97, 120),
				"Bologna Centrale", new Station("Bologna Centrale", 0, 120),
				"Giulianova", new Station("Giulianova", 312.36, 120)
				),
				new TreeSet<>(Set.of("Bologna Centrale")
				));
		RailwayLine lineaBoMi = new RailwayLine(Map.of(
				"Modena", new Station("Modena", 36.93, 160),
				"Reggio Emilia", new Station("Reggio Emilia", 61.44, 160),
				"Parma", new Station("Parma", 89.74, 160),
				"Bologna Centrale", new Station("Bologna Centrale", 0.0, 160),
				"Piacenza", new Station("Piacenza", 146.82, 160)
				),
				new TreeSet<>(Set.of("Bologna Centrale", "Parma", "Modena")
				));
		
		RouteFinder finder = new ClassicRouteFinder(Set.of(lineaBoMi,lineaBoLe));
		
		assertEquals(List.of(
					new Route(new Segment(lineaBoMi, new Station("Reggio Emilia", 61.44, 160), new Station("Bologna Centrale", 0.0, 160))),
					new Route(
							new Segment(lineaBoMi, new Station("Reggio Emilia", 61.44, 160), new Station("Modena", 36.93, 160)),
							new Segment(lineaBoMi, new Station("Modena", 36.93, 160), new Station("Bologna Centrale", 0.0, 160))
					),
					new Route(
							new Segment(lineaBoMi, new Station("Reggio Emilia", 61.44, 160), new Station("Parma", 89.74, 160)),
							new Segment(lineaBoMi, new Station("Parma", 89.74, 160),new Station("Bologna Centrale", 0.0, 160))
					)
				),
				finder.getRoutes("Reggio Emilia", "Bologna Centrale"));
		
		assertEquals(Collections.emptyList(), finder.getRoutes("XXXX", "Bologna Centrale"));
		assertEquals(Collections.emptyList(), finder.getRoutes("Reggio Emilia", "YYY"));
		assertEquals(Collections.emptyList(), finder.getRoutes("Reggio Emilia", ""));
		assertEquals(Collections.emptyList(), finder.getRoutes("", "Bologna Centrale"));
		assertEquals(Collections.emptyList(), finder.getRoutes(null, "Bologna Centrale"));
		//assertThrows(IllegalArgumentException.class, () -> finder.getRoutes("Reggio Emilia", null));
		assertEquals(Collections.emptyList(), finder.getRoutes("Reggio Emilia", null));
		

		assertEquals(List.of(new Route(new Segment(lineaBoLe,new Station("Giulianova", 312.36, 120), new Station("Bologna Centrale", 0, 120)))),
				finder.getRoutes("Giulianova", "Bologna Centrale"));
				
		assertEquals(Set.of(lineaBoMi), finder.getLinesAtStation("Reggio Emilia"));
		assertEquals(Set.of(lineaBoMi), finder.getLinesAtStation("Piacenza"));
		assertEquals(Set.of(lineaBoMi,lineaBoLe), finder.getLinesAtStation("Bologna Centrale"));
		assertEquals(Set.of(lineaBoLe), finder.getLinesAtStation("Giulianova"));
		assertEquals(Collections.emptySet(), finder.getLinesAtStation("XXX"));
		assertEquals(Collections.emptySet(), finder.getLinesAtStation(null));
	}
	
	@Test
	void testOKOneRouteWithChange() {
		RailwayLine lineaBoLe = new RailwayLine(Map.of(
				"Santarcangelo di Romagna", new Station("Santarcangelo di Romagna", 101.27, 120),
				"RiminiFiera", new Station("RiminiFiera", 106.97, 120),
				"Bologna Centrale", new Station("Bologna Centrale", 0, 120),
				"Giulianova", new Station("Giulianova", 312.36, 120)
				),
				new TreeSet<>(Set.of("Bologna Centrale")
				));
		RailwayLine lineaBoMi = new RailwayLine(Map.of(
				"Modena", new Station("Modena", 36.93, 160),
				"Reggio Emilia", new Station("Reggio Emilia", 61.44, 160),
				"Parma", new Station("Parma", 89.74, 160),
				"Bologna Centrale", new Station("Bologna Centrale", 0.0, 160),
				"Piacenza", new Station("Piacenza", 146.82, 160)
				),
				new TreeSet<>(Set.of("Bologna Centrale", "Parma", "Modena")
				));
		
		RouteFinder finder = new ClassicRouteFinder(Set.of(lineaBoMi,lineaBoLe));
		
		assertEquals(List.of(new Route(
				new Segment(lineaBoMi, new Station("Modena", 36.93, 160), new Station("Bologna Centrale", 0.0, 160)),
				new Segment(lineaBoLe, new Station("Bologna Centrale", 0.0, 120), new Station("Giulianova", 312.36, 120))
				)),
				finder.getRoutes("Modena","Giulianova"));
		
		assertEquals(List.of(new Route(
				new Segment(lineaBoMi, new Station("Reggio Emilia", 61.44, 160), new Station("Bologna Centrale", 0.0, 160)),
				new Segment(lineaBoLe, new Station("Bologna Centrale", 0.0, 120), new Station("Giulianova", 312.36, 120))
				)),
				finder.getRoutes("Reggio Emilia","Giulianova"));
	}
	
	@Test
	void testOKTwoRoutesWithChange() {
		RailwayLine lineaBoVr = new RailwayLine(Map.of(
				"Verona Porta Nuova", new Station("Verona Porta Nuova", 114.95, 200),
				"Bologna Centrale", new Station("Bologna Centrale", 0.0, 200)
				),
				new TreeSet<>(Set.of("Bologna Centrale", "Verona Porta Nuova")
				));
		RailwayLine lineaMoVr = new RailwayLine(Map.of(
				"Verona Porta Nuova", new Station("Verona Porta Nuova", 97.86, 110),
				"Modena", new Station("Modena", 0.0, 110)
				),
				new TreeSet<>(Set.of("Modena", "Verona Porta Nuova")
				));
		RailwayLine lineaBoMi = new RailwayLine(Map.of(
				"Modena", new Station("Modena", 36.93, 160),
				"Reggio Emilia", new Station("Reggio Emilia", 61.44, 160),
				"Parma", new Station("Parma", 89.74, 160),
				"Bologna Centrale", new Station("Bologna Centrale", 0.0, 160),
				"Piacenza", new Station("Piacenza", 146.82, 160)
				),
				new TreeSet<>(Set.of("Bologna Centrale", "Parma", "Modena")
				));
		
		RouteFinder finder = new ClassicRouteFinder(Set.of(lineaBoMi,lineaBoVr,lineaMoVr));
		
		assertTrue(List.of(
					new Route(
							new Segment(lineaBoMi, new Station("Reggio Emilia", 61.44, 160), new Station("Modena", 36.93, 160)),
							new Segment(lineaMoVr, new Station("Modena", 0.000, 110), new Station("Verona Porta Nuova", 97.86, 110))
				),
				new Route(
							new Segment(lineaBoMi, new Station("Reggio Emilia", 61.44, 160), new Station("Bologna Centrale", 0.00, 160)),
							new Segment(lineaBoVr, new Station("Bologna Centrale", 0.0, 200), new Station("Verona Porta Nuova", 114.95, 200))
					)).containsAll(
					finder.getRoutes("Reggio Emilia","Verona Porta Nuova")));
		
		assertEquals(Set.of(lineaBoMi), finder.getLinesAtStation("Reggio Emilia"));
		assertEquals(Set.of(lineaBoMi), finder.getLinesAtStation("Piacenza"));
		assertEquals(Set.of(lineaBoMi,lineaBoVr), finder.getLinesAtStation("Bologna Centrale"));
		assertEquals(Set.of(lineaBoMi,lineaMoVr), finder.getLinesAtStation("Modena"));
		assertEquals(Set.of(lineaMoVr,lineaBoVr), finder.getLinesAtStation("Verona Porta Nuova"));
		assertEquals(Collections.emptySet(), finder.getLinesAtStation("XXX"));
		assertEquals(Collections.emptySet(), finder.getLinesAtStation(null));
	}
	
	@Test
	void testOKRoundTrip() {
		RailwayLine lineaBoLe = new RailwayLine(Map.of(
				"Santarcangelo di Romagna", new Station("Santarcangelo di Romagna", 101.27, 120),
				"RiminiFiera", new Station("RiminiFiera", 106.97, 120),
				"Bologna Centrale", new Station("Bologna Centrale", 0, 120),
				"Giulianova", new Station("Giulianova", 312.36, 120)
				),
				new TreeSet<>(Set.of("Bologna Centrale")
				));
		RailwayLine lineaBoMi = new RailwayLine(Map.of(
				"Modena", new Station("Modena", 36.93, 160),
				"Reggio Emilia", new Station("Reggio Emilia", 61.44, 160),
				"Parma", new Station("Parma", 89.74, 160),
				"Bologna Centrale", new Station("Bologna Centrale", 0.0, 160),
				"Piacenza", new Station("Piacenza", 146.82, 160)
				),
				new TreeSet<>(Set.of("Bologna Centrale", "Parma", "Modena")
				));
		
		RouteFinder finder = new ClassicRouteFinder(Set.of(lineaBoMi,lineaBoLe));
		
		assertTrue(List.of(
				new Route(
						new Segment(lineaBoMi, new Station("Modena", 36.93, 160), new Station("Bologna Centrale", 0.0, 160)),
						new Segment(lineaBoMi, new Station("Bologna Centrale", 0.0, 160),new Station("Modena", 36.93, 160))
				),
				new Route(
						new Segment(lineaBoMi, new Station("Modena", 36.93, 160), new Station("Parma", 89.74, 160)),
						new Segment(lineaBoMi, new Station("Parma", 89.74, 160), new Station("Modena", 36.93, 160))
				)).containsAll(
				finder.getRoutes("Modena","Modena")));
		
		assertTrue(List.of(
				new Route(
						new Segment(lineaBoMi, new Station("Modena", 36.93, 160), new Station("Bologna Centrale", 0.0, 160)),
						new Segment(lineaBoMi, new Station("Bologna Centrale", 0.0, 160), new Station("Modena", 36.93, 160))
				),
				new Route(
						new Segment(lineaBoMi, new Station("Modena", 36.93, 160), new Station("Parma", 89.74, 160)),
						new Segment(lineaBoMi, new Station("Parma", 89.74, 160), new Station("Modena", 36.93, 160))
				)).containsAll(
				finder.getRoutes("Modena","Modena")));

	}

}


