package rfd.model.routefinder.test;

import static org.junit.jupiter.api.Assertions.*;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import org.junit.jupiter.api.Test;

import rfd.model.Station;
import rfd.model.routefinder.RouteFinder;
import rfd.model.RailwayLine;
import rfd.model.Route;

public class RouteFinderTest {

	@Test
	void testOK() {
		RailwayLine lineaBoLe = new RailwayLine(Map.of(
				"Santarcangelo di Romagna", new Station("Santarcangelo di Romagna", 101.27, 120),
				"RiminiFiera", new Station("RiminiFiera", 106.97, 120),
				"Bologna Centrale", new Station("Bologna Centrale", 0, 120),
				"Giulianova", new Station("Giulianova", 312.36, 120)
				),
				new TreeSet<>(Set.of("Bologna Centrale")
				));
		RailwayLine lineaBoMi = new RailwayLine(Map.of(
				"Modena", new Station("Modena", 36.93, 160),
				"Reggio Emilia", new Station("Reggio Emilia", 61.44, 160),
				"Parma", new Station("Parma", 89.74, 160),
				"Bologna Centrale", new Station("Bologna Centrale", 0.0, 160),
				"Piacenza", new Station("Piacenza", 146.82, 160)
				),
				new TreeSet<>(Set.of("Bologna Centrale", "Parma", "Modena")
				));
		
		RouteFinder finder = new RouteFinder(Set.of(lineaBoMi,lineaBoLe)) {
			public List<Route> getRoutes(String from, String to) {
				return Collections.emptyList();
			}
		};
		// test getRoutes
		assertEquals(Collections.emptyList(), finder.getRoutes("Reggio Emilia", "Bologna Centrale"));
		assertEquals(Collections.emptyList(), finder.getRoutes("XXXX", "Bologna Centrale"));
		assertEquals(Collections.emptyList(), finder.getRoutes("Reggio Emilia", "YYY"));
		assertEquals(Collections.emptyList(), finder.getRoutes("Reggio Emilia", ""));
		assertEquals(Collections.emptyList(), finder.getRoutes("", "Bologna Centrale"));
		// the default fake finder never throws exceptions
		assertEquals(Collections.emptyList(), finder.getRoutes("Reggio Emilia", null));
		assertEquals(Collections.emptyList(), finder.getRoutes(null, "Bologna Centrale"));
		// test getLinesAtStation
		assertEquals(Set.of(lineaBoMi), finder.getLinesAtStation("Reggio Emilia"));
		assertEquals(Set.of(lineaBoMi), finder.getLinesAtStation("Piacenza"));
		assertEquals(Set.of(lineaBoLe), finder.getLinesAtStation("RiminiFiera"));
		assertEquals(Set.of(lineaBoMi,lineaBoLe), finder.getLinesAtStation("Bologna Centrale"));
		assertEquals(Collections.emptySet(), finder.getLinesAtStation("XXX"));
		assertEquals(Collections.emptySet(), finder.getLinesAtStation(null));
	}

}
